package com.warewms.ams.ndc.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.comparator.CompareUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.warewms.ams.common.AmsConstant;
import com.warewms.ams.common.SceneConstants;
import com.warewms.ams.ndc.common.ByteUtil;
import com.warewms.ams.ndc.common.CRC16Util;
import com.warewms.ams.ndc.domain.AmsTask;
import com.warewms.ams.ndc.domain.BaseLocationInfo;
import com.warewms.ams.ndc.dto.AmsTaskCancelDTO;
import com.warewms.ams.ndc.dto.AmsTaskDTO;
import com.warewms.ams.ndc.dto.AmsTaskInfoDTO;
import com.warewms.ams.ndc.dto.AmsTaskInsertDTO;
import com.warewms.ams.ndc.mapper.AmsTaskMapper;
import com.warewms.ams.ndc.service.IAmsTaskService;
import com.warewms.ams.ndc.service.IBaseLocationInfoService;
import com.warewms.common.exception.base.BaseException;
import com.warewms.common.utils.ConvertUtils;
import com.warewms.common.utils.redis.RedisCache;
import com.warewms.common.utils.string.StringUtils;
import com.warewms.framework.service.impl.CrudServiceImpl;
import com.warewms.system.service.ISysConfigService;
import io.jsonwebtoken.lang.Assert;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * ams任务列表Service业务层处理
 * 
 * @author andy
 * @date 2022-08-08
 */
@Service
public class AmsTaskServiceImpl extends CrudServiceImpl<AmsTaskMapper, AmsTask, AmsTaskDTO> implements IAmsTaskService
{
    @Autowired
    private AmsTaskMapper amsTaskMapper;

    @Autowired
    private IBaseLocationInfoService baseLocationInfoService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ISysConfigService sysConfigService;

    public static int genIKey(String taskNo) {
        int res = CRC16Util.calcCrc16(ByteUtil.string2byteArray(taskNo));
        if (res == 0) {
            BigDecimal t = new BigDecimal(taskNo);
            t = t.add(BigDecimal.ONE);
            return genIKey(t.toString());
        } else {
            return res;
        }
    }

    /**
     * 查询ams任务列表
     * 
     * @param id ams任务列表主键
     * @return ams任务列表
     */
    @Override
    public AmsTask selectAmsTaskById(Long id)
    {
        return amsTaskMapper.selectAmsTaskById(id);
    }

    /**
     * 查询ams任务列表列表
     * 
     * @param amsTask ams任务列表
     * @return ams任务列表
     */
    @Override
    public List<AmsTask> selectAmsTaskList(AmsTask amsTask)
    {
        return amsTaskMapper.selectAmsTaskList(amsTask);
    }

    /**
     * 查询ams任务列表
     *
     * @param amsTask ams任务列表
     * @return ams任务列表
     */
    @Override
    public AmsTask selectAmsTaskByModel(AmsTask amsTask)
    {
        List<AmsTask> list = amsTaskMapper.selectAmsTaskList(amsTask);
        if (list!=null && list.size()>0) {
            return list.get(0);
        } else {
            return null;
        }
    }

    /**
     * 新增ams任务列表
     * 
     * @param amsTask ams任务列表
     * @return 结果
     */
    @Override
    public int insertAmsTask(AmsTask amsTask)
    {
        return amsTaskMapper.insertAmsTask(amsTask);
    }

    /**
     * 修改ams任务列表
     * 
     * @param amsTask ams任务列表
     * @return 结果
     */
    @Override
    public int updateAmsTask(AmsTask amsTask)
    {
        return amsTaskMapper.updateAmsTask(amsTask);
    }

    /**
     * 批量删除ams任务列表
     * 
     * @param ids 需要删除的ams任务列表主键
     * @return 结果
     */
    @Override
    public int deleteAmsTaskByIds(Long[] ids)
    {
        return amsTaskMapper.deleteAmsTaskByIds(ids);
    }

    /**
     * 删除ams任务列表信息
     * 
     * @param id ams任务列表主键
     * @return 结果
     */
    @Override
    public int deleteAmsTaskById(Long id)
    {
        return amsTaskMapper.deleteAmsTaskById(id);
    }

    @Override
    public List<AmsTask> selectTaskNotIssued() {
        return amsTaskMapper.selectTaskNotIssued();
    }

    /**
     * 下发任务
     * @param amsTaskInsertDTO
     * @return
     */
    @Override
    public Long amsTaskInsert(AmsTaskInsertDTO amsTaskInsertDTO) {
        List<AmsTask> wcsTaskList = amsTaskMapper.selectList(Wrappers.<AmsTask>lambdaQuery().eq(AmsTask::getTaskNo, amsTaskInsertDTO.getBusinessNo()));
        Assert.isTrue(CollectionUtil.isEmpty(wcsTaskList), "任务下发重复！");
        AmsTask amsTask = ConvertUtils.sourceToTarget(amsTaskInsertDTO, AmsTask.class);
        amsTask.setId(Long.parseLong(IdWorker.getIdStr()));
        amsTask.setTaskNo(amsTaskInsertDTO.getBusinessNo());
        amsTask.setStFrom(convertLocation(amsTaskInsertDTO.getLocationFrom()));
        boolean isExist = redisCache.checkIsExist(AmsConstant.WMS_CORRESPOND_AGV_STATION_KEY.concat(amsTaskInsertDTO.getLocationFrom()))
                && redisCache.checkIsExist(AmsConstant.WMS_CORRESPOND_AGV_STATION_KEY.concat(amsTaskInsertDTO.getLocationTo()));
        amsTask.setStTo(isExist ? convertLocation(amsTaskInsertDTO.getLocationTo()) : getCacheLocation(amsTaskInsertDTO.getLocationFrom(), amsTaskInsertDTO.getLocationTo()));
        amsTask.setExt3(isExist ? null : amsTaskInsertDTO.getLocationTo());
        amsTask.setExt1(isExist ? redisCache.getCacheObject(AmsConstant.LOCATION_ROW_KEY.concat(amsTask.getStTo().toString())) : String.valueOf(BigDecimal.ZERO));
        amsTask.setExt2(amsTask.getExt1());
        amsTask.setBusinessType(StringUtils.isNotBlank(amsTaskInsertDTO.getBusinessType()) ?
                amsTaskInsertDTO.getBusinessType() : AmsConstant.TASK_BUSINESS_TYPE.TASK_01.getValue());
        amsTask.setIsDelete(0);
        amsTask.setAciAccept(0);
        amsTask.setIkey((long) genIKey(amsTask.getId().toString()));
        amsTask.setRemark("move the start location to the cache location");
        amsTaskMapper.insert(amsTask);
        return amsTask.getId();
    }

    @Override
    public void ndcAmsTaskInsert(String businessNo, String businessType, Integer stFrom, String stTo) {
        AmsTask amsTask = new AmsTask();
        amsTask.setId(Long.parseLong(IdWorker.getIdStr()));
        amsTask.setTaskNo(businessNo);
        amsTask.setStFrom(convertLocation(stFrom.toString()));
        amsTask.setStTo(convertLocation(stTo));
        amsTask.setExt1(redisCache.getCacheObject(AmsConstant.LOCATION_ROW_KEY.concat(stTo)));
        amsTask.setExt2(amsTask.getExt1());
        amsTask.setBusinessType(businessType);
        amsTask.setIsDelete(0);
        amsTask.setAciAccept(0);
        amsTask.setIkey((long) genIKey(amsTask.getId().toString()));
        amsTask.setRemark("move the cache location to the end location");
        amsTaskMapper.insert(amsTask);
    }

    private Integer convertLocation(String locationNo) {
        boolean isExist = redisCache.checkIsExist(AmsConstant.WMS_CORRESPOND_AGV_STATION_KEY.concat(locationNo));
        return isExist ? Integer.parseInt(redisCache.getCacheObject(AmsConstant.WMS_CORRESPOND_AGV_STATION_KEY.concat(locationNo)).toString()) : Integer.parseInt(locationNo);
    }

    private Integer getCacheLocation(String locationFromId, String locationToId) {
        boolean isExist = redisCache.checkIsExist(AmsConstant.WMS_CORRESPOND_AGV_STATION_KEY.concat(locationFromId));
        String rowNo = isExist ?
                redisCache.getCacheObject(AmsConstant.LOCATION_ROW_KEY.concat(locationFromId)) : redisCache.getCacheObject(AmsConstant.LOCATION_ROW_KEY.concat(locationToId));
        String locationConfig = sysConfigService.selectConfigByKey(SceneConstants.LOCATION_SCENE);
        if(StringUtils.isBlank(locationConfig)) throw new BaseException("location.scene.config is blank");
        Map<String, List<Integer>> locationConfigMap = JSONObject.parseObject(locationConfig, Map.class);
        String sceneValueKey = locationConfigMap.entrySet().stream().filter(item -> item.getValue().contains(Integer.valueOf(isExist ? locationToId : locationFromId))).map(Map.Entry::getKey).findFirst().orElseGet(() -> null);
        Integer zoneId = SceneConstants.locationToAreaCacheListMapper.get(sceneValueKey);
        Assert.isTrue(ObjectUtil.isNotNull(zoneId), "location.scene.config is not exist this locationId");
        List<BaseLocationInfo> baseLocationInfoList = baseLocationInfoService.queryBaseLocationInfoList(zoneId, rowNo);
        Assert.isTrue(CollectionUtil.isNotEmpty(baseLocationInfoList), "this zone cache location is not exist");
        return Integer.parseInt(baseLocationInfoList.stream().findFirst().orElseThrow(() -> new BaseException("this zone cache location is not exist")).getLocationNo());
    }

    /**
     * 获取AMS任务详情
     * @param businessNo
     * @return
     */
    @Override
    public AmsTaskInfoDTO getAmsTaskInfo(String businessNo) {
        List<AmsTask> amsTaskList = amsTaskMapper.selectList(Wrappers.<AmsTask>lambdaQuery().eq(AmsTask::getTaskNo, businessNo));
        AmsTask amsTask = amsTaskList.stream().findFirst().orElseThrow(() -> new BaseException("任务编号有误！"));
        AmsTaskInfoDTO amsTaskInfoDTO = ConvertUtils.sourceToTarget(amsTask, AmsTaskInfoDTO.class);
        amsTaskInfoDTO.setState(getAmsTaskState(amsTask));
        return amsTaskInfoDTO;
    }

    /**
     * 任务取消
     * @param amsTaskCancelDTO
     * @return
     */
    @Transactional
    @Override
    public Boolean cancelAmsTask(AmsTaskCancelDTO amsTaskCancelDTO) {
        String id = amsTaskCancelDTO.getTaskNo();
        String remark = StringUtils.isNotBlank(amsTaskCancelDTO.getReason()) ? amsTaskCancelDTO.getReason() : null;
        AmsTask amsTask = getAmsTaskById(id);
        Assert.isTrue(ObjectUtil.isNotNull(amsTask), "任务不存在！");
        //请求ams 取消任务
        Assert.isTrue(ObjectUtil.isNull(amsTask.getEvent()) || CompareUtil.compare(amsTask.getEvent(), AmsConstant.TASK_STS.TASK7.getKey()) != 0
                , "任务已取消！");
        return amsTaskMapper.update(new AmsTask(), Wrappers.<AmsTask>lambdaUpdate()
                .eq(AmsTask::getId, id)
                .set(AmsTask::getIsDelete, AmsConstant.DELETE_STATUS.Y.getValue())
                .set(AmsTask::getAciAccept, AmsConstant.ACI_ACCEPT_STATUS.N.getValue())
                .set(AmsTask::getRemark, "下发超时1分钟未接收自动取消, 取消原因为：" + remark)) > 0;
    }

    public Integer getAmsTaskState(AmsTask amsTask){
        Integer delete = ObjectUtil.isNotNull(amsTask.getIsDelete()) ? amsTask.getIsDelete() : null;
        Integer aciAccept = ObjectUtil.isNotNull(amsTask.getAciAccept()) ? amsTask.getAciAccept() : null;
        if ((CompareUtil.compare(new BigDecimal(aciAccept), BigDecimal.ZERO) == 0 &&
                CompareUtil.compare(new BigDecimal(delete), BigDecimal.ZERO) == 0) || ObjectUtil.isNull(amsTask.getAciIndex()) || ObjectUtil.isNull(amsTask.getEvent())){
            return AmsConstant.TASK_STS.TASK0.getValue();
        }
        return AmsConstant.TASK_STS.getValueByKey(amsTask.getEvent());
    }

    public AmsTask getAmsTaskById(String id){
        return getAmsTaskList(null, Lists.newArrayList(id), null).stream().findFirst().orElseGet(() -> null);
    }

    @Override
    public AmsTask getAmsTaskByAciIndex(Integer aciIndex){
        return getAmsTaskList(null, null, aciIndex).stream().findFirst().orElseGet(() -> null);
    }

    public List<AmsTask> getAmsTaskList(List<String> businessNoList, List<String> idList, Integer aciIndex){
        List<AmsTask> amsTaskList = amsTaskMapper.selectList(Wrappers.<AmsTask>lambdaQuery()
                .in(CollectionUtil.isNotEmpty(businessNoList), AmsTask::getTaskNo, businessNoList)
                .in(CollectionUtil.isNotEmpty(idList), AmsTask::getId, idList)
                .eq(ObjectUtil.isNotNull(aciIndex), AmsTask::getAciIndex, aciIndex)
                .orderByDesc(AmsTask::getAddtime));
        return amsTaskList;
    }

    @Override
    public QueryWrapper<AmsTask> getWrapper(Map<String, Object> params) {
        QueryWrapper<AmsTask> queryWrapper = new QueryWrapper<>();
        String taskNo = (String) params.get("taskNo");
        String aciIndex = (String) params.get("aciIndex");
        String stFrom = (String) params.get("stFrom");
        String stTo = (String) params.get("stTo");
        queryWrapper.lambda().eq(StringUtils.isNotBlank(taskNo), AmsTask::getTaskNo, Integer.parseInt(taskNo))
                .eq(StringUtils.isNotBlank(aciIndex), AmsTask::getAciIndex, Integer.parseInt(aciIndex))
                .eq(StringUtils.isNotBlank(stFrom), AmsTask::getStFrom, Integer.parseInt(stFrom))
                .eq(StringUtils.isNotBlank(stTo), AmsTask::getStTo, Integer.parseInt(stTo));
        return queryWrapper;
    }
}
